import io
import datetime

from django.conf import settings
from django.utils._os import safe_join
from django.core.files.storage import Storage
from django.core.exceptions import ImproperlyConfigured

from dateutil import parser, tz

import requests


class BunnyStorage(Storage):
    """
    Implementation of Django's storage module using Bunny.net.
    """

    username = None
    password = None
    region = None
    hostname = None

    base_url = None
    headers = None

    def __init__(self, **kwargs):
        username = (
            kwargs["username"] if "username" in kwargs else settings.BUNNY_USERNAME
        )
        password = (
            kwargs["password"] if "password" in kwargs else settings.BUNNY_PASSWORD
        )
        region = (
            kwargs["region"]
            if "region" in kwargs
            else settings.BUNNY_REGION
            if settings.BUNNY_REGION
            else "ny"
        )
        hostname = (
            kwargs["hostname"]
            if "hostname" in kwargs
            else settings.BUNNY_HOSTNAME
            if settings.BUNNY_HOSTNAME
            else settings.MEDIA_URL
        )

        if not username:
            raise ImproperlyConfigured(
                "Setting BUNNY_USERNAME or `username` option is required."
            )

        if not password:
            raise ImproperlyConfigured(
                "Setting BUNNY_PASSWORD or `password` option is required."
            )

        self.base_url = ""

        if region == "de":
            self.base_url += "https://storage.bunnycdn.com/"
        else:
            self.base_url += f"https://{region}.storage.bunnycdn.com/"

        self.username = username
        self.password = password
        self.region = region
        self.hostname = hostname

        self.base_url += f"{username}/"
        self.headers = {"AccessKey": password}

    def _full_path(self, name) -> str:
        if name == "/":
            name = ""
        return safe_join(self.base_url, name).replace("\\", "/")

    def _save(self, name, content) -> str:
        r = requests.put(
            safe_join(self.base_url, name), data=content, headers=self.headers
        )
        return name

    def _open(self, name, mode="rb"):
        r = requests.get(safe_join(self.base_url, name), headers=self.headers)
        r.raise_for_status()

        return r.raw

    def delete(self, name) -> str:
        r = requests.delete(safe_join(self.base_url, name), headers=self.headers)

        return name

    def exists(self, name) -> bool:
        r = requests.head(safe_join(self.base_url, name), headers=self.headers)

        if r.status_code == 404:
            return False

        r.raise_for_status()

        return True

    def url(self, name: str) -> str:
        return safe_join(self.hostname, name)

    def size(self, name: str) -> int:
        r = requests.head(self.url(name))
        r.raise_for_status()

        file_size = r.headers.get("Content-Length", 0)

        return int(file_size)

    def listdir(self, path) -> tuple:
        r = requests.get(safe_join(self.base_url, path), headers=self.headers)
        r.raise_for_status()

        objects = r.json()

        return (
            [item["ObjectName"] for item in objects if item["IsDirectory"]],
            [item["ObjectName"] for item in objects if not item["IsDirectory"]],
        )

    def get_created_time(self, name) -> datetime.datetime:
        r = requests.get(
            safe_join(self.base_url, name.rsplit("/", 1)[0]), headers=self.headers
        )
        r.raise_for_status()

        iso_date = None

        for item in r.json():
            if item["ObjectName"] == name:
                iso_date = item["DateCreated"]
                break

        if iso_date is None:
            raise ValueError(f"File '{name}' does not exist.")

        if settings.USE_TZ == True:
            parsed_date = parser.parse(iso_date)
            parsed_date.replace(tzinfo=datetime.timezone.utc)
            return parsed_date
        else:
            parsed_date = parser.parse(iso_date)
            parsed_date.replace(tzinfo=datetime.timezone.utc)
            parsed_date.astimezone()
            parsed_date.replace(tzinfo=None)
            return parsed_date

    def get_modified_time(self, name) -> datetime.datetime:
        r = requests.get(
            safe_join(self.base_url, name.rsplit("/", 1)[0]), headers=self.headers
        )
        r.raise_for_status()

        iso_date = None

        for item in r.json():
            if item["ObjectName"] == name.rsplit("/", 1)[1]:
                iso_date = item["LastChanged"]
                break

        if iso_date is None:
            raise ValueError(f"File '{name}' does not exist.")

        if settings.USE_TZ == True:
            parsed_date = parser.parse(iso_date)
            parsed_date.replace(tzinfo=datetime.timezone.utc)
            return parsed_date
        else:
            parsed_date = parser.parse(iso_date)
            parsed_date.replace(tzinfo=datetime.timezone.utc)
            parsed_date.astimezone()
            parsed_date.replace(tzinfo=None)
            return parsed_date



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_bunny']

package_data = \
{'': ['*']}

install_requires = \
['Django>=3.0.0', 'python-dateutil>=2.8.2,<3.0.0', 'requests>=2.30.0,<3.0.0']

setup_kwargs = {
    'name': 'django-bunny',
    'version': '1.0.2',
    'description': 'A django storage for bunny.net',
    'long_description': '# django-bunny\n\nA bunny.net storage for Django. It was created as a replacement for [django-bunny-storage](https://github.com/willmeyers/django-bunny-storage).\n\n\n## Installation\n\nYou can install the library using `pip`:\n\n```bash\npip install django-bunny\n```\n\n\n## Configuration\n\nFirst, add `django_bunny` to your `INSTALLED_APPS`:\n\n```py\nINSTALLED_APPS = [\n    ...,\n    "django_bunny",\n    ...\n]\n```\n\nNow, create the following variables inside your `settings.py` file. These are required if you are using Django < `4.2`. Otherwise, you can use `OPTIONS` in the `STORAGES` setting.\n\n```py\n# These can be found in your storage\'s dashboard under `FTP & API Access`\nBUNNY_USERNAME = "my-storage-name"\nBUNNY_PASSWORD = "my-storage-password"\n\n# This is the storage region\'s code. E.g. Los Angeles is `la`, Singapore is\n# `sg`, etc. The default is `ny` (New York).\nBUNNY_REGION = "my-storage-region"\n\n# Optional. For example, `https://myzone.b-cdn.net/`. `MEDIA_URL` will be used\n# if this is not set.\nBUNNY_HOSTNAME = "my-pullzone-hostname"\n```\n\nFinally, depending on which version of Django you are using you\'ll need to create one of these variables:\n\n```py\n# Django < 4.2\nDEFAULT_FILE_STORAGE = \'django_bunny.storage.BunnyStorage\'\n\n# Django >= 4.2. Set `BunnyStorage` where you want to use bunny.net\'s storage.\n{\n    "default": {\n        "BACKEND": "django_bunny.storage.BunnyStorage",\n\n        # Add this if you did not create the BUNNY_* settings before. They are\n        # the same as BUNNY_* variables.\n        "OPTIONS": {\n            "username": "my-storage-name",\n            "password": "my-storage-password",\n            \n            # Optional. Defaults to `ny`\n            "region": "my-storage-region",\n            \n            # Optional. `MEDIA_URL` will be used if it is not set\n            "hostname": "my-pullzone-hostname"\n        }\n    },\n}\n```\n\nIn Django >= 4.2 you can set different credentials for the different storages. This allows you to, for example, use a separate storage for static files.\n\n\n## Using in templates\n\nIn templates, you can use `{{ instance.file.url }}` directly. For example:\n\n```html\n<img src="{{ instance.file.url }}" />\n```\n',
    'author': 'Neki',
    'author_email': 'neki@nekidev.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
